using System;
using System.Data;
using System.Data.SqlClient;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.Management.Smo;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SyncDBMSSQL.Properties;

namespace SyncDBMSSQL {
    class DataAccessLayer : IDisposable {
        private string _connectionString = "";
        private SqlConnection conn;

        SqlConnectionStringBuilder csb = null;
        Server server = null;
        Database sqlDB = null;

        private Queue<Table> orderTables = new Queue<Table>();
        private Hashtable restrictTableList = new Hashtable();
        private Hashtable uniqueTableList = new Hashtable();

        #region Constructores
        public DataAccessLayer() {
        }

        public DataAccessLayer(string connectionString) {
            this._connectionString = connectionString;
            this.OpenConnection();

            this.csb = new SqlConnectionStringBuilder(connectionString);
            server = new Server(new ServerConnection(this.conn));
            sqlDB = server.Databases[csb.InitialCatalog];
        }
        #endregion

        public void OpenConnection() {
            this.conn = new SqlConnection(this._connectionString);
            this.conn.Open();

            this.csb = new SqlConnectionStringBuilder(this._connectionString);
            server = new Server(new ServerConnection(this.conn));
            sqlDB = server.Databases[csb.InitialCatalog];
        }

        public void OpenConnection(string connectionString) {
            this._connectionString = connectionString;
            this.OpenConnection();

            this.csb = new SqlConnectionStringBuilder(connectionString);
            server = new Server(new ServerConnection(this.conn));
            sqlDB = server.Databases[csb.InitialCatalog];
        }

        public void CloseConnection() {
            if (conn.State == ConnectionState.Open)
                conn.Close();
        }

        public List<string> getTables() {
            List<string> tablas = new List<string>();

            // Inicializa la conexin al servidor sql server
            foreach (Table tbl in sqlDB.Tables) {
                if (!tbl.IsSystemObject) tablas.Add(string.Format("{0}.{1}", tbl.Schema, tbl.Name));
            }
            return tablas;
        }

        public bool ExisteTabla(string tableName) {
            // Inicializa la conexin al servidor sql server
            return (sqlDB.Tables.Contains(tableName.Substring(tableName.IndexOf(".") + 1)));
        }

        public List<TableColumnInfo> getTableColumns(string tableName) {
            if (tableName.IndexOf(".") >= 0) tableName = tableName.Substring(tableName.IndexOf(".") + 1);

            Table table = getTableMetadata(tableName);
            List<TableColumnInfo> cols = new List<TableColumnInfo>();
            foreach (Column col in table.Columns) {
                if (!col.Computed) {
                    cols.Add(new TableColumnInfo(col.Name, col.DataType.SqlDataType.ToString(), col.InPrimaryKey));
                }
            }
            return cols;
        }

        private Table getTableMetadata(string tableName) {
            if (ExisteTabla(tableName))
                return sqlDB.Tables[tableName];
            else
                return null;
        }

        public bool TableHasIdentityColumn(string tableName) {
            if (tableName.IndexOf(".") >= 0) tableName = tableName.Substring(tableName.IndexOf(".") + 1);

            Table tbl = getTableMetadata(tableName);
            foreach (Column col in tbl.Columns)
                if (col.Identity) return true;

            return false;
        }

        public string generateInsert(string tableName, List<TableColumnInfo> columnas, string where, string valores) {
            string tblName = tableName.Substring(tableName.IndexOf(".") + 1);
            string tblSchema = tableName.Substring(0, tableName.IndexOf("."));

            string cols = this.ColumnList(columnas);
            DataSet ds = this.ExecuteDataSet(string.Format("select {0} from [{1}].[{2}] where {3} = '{4}'", cols, tblSchema, tblName, where, valores));
            string vals = "";
            DataRow dr = ds.Tables[0].Rows[0];
            foreach (TableColumnInfo info in columnas) {
                vals = vals + this.formatValue(dr, info) + ", ";
            }
            vals = vals.Substring(0, vals.Length - 2);
            return string.Format("insert into [{0}].[{1}] ({2}) values ({3}){4}", tblSchema, tblName, cols, vals, SyncDBMSSQL.Properties.Settings.Default.EndOfLineCharacter);
        }

        public DataRow getDataRow(string tableName, List<TableColumnInfo> columnas, string where, string valores) {
            string tblName = tableName.Substring(tableName.IndexOf(".") + 1);
            string tblSchema = tableName.Substring(0, tableName.IndexOf("."));

            string cols = this.ColumnList(columnas);
            DataSet ds = this.ExecuteDataSet(string.Format("select {0} from [{1}].[{2}] where {3} = '{4}'", cols, tblSchema, tblName, where, valores));
            if (ds != null && ds.Tables.Count > 0 && ds.Tables[0].Rows.Count > 0) {
                return ds.Tables[0].Rows[0];
            }
            return null;
        }

        public DataSet ExecuteDataSet(string sqlStatement) {
            return this.ExecuteDataSet("DataSet", sqlStatement);
        }

        /// <summary> Retorna un DataSet a partir de una instruccin sql</summary>
        /// <param name="dataSetName">Nombre del DataSet</param>
        /// <param name="sqlStatement">Instruccin SQL a ejecutar</param>
        /// <returns>Un DataSet con los datos que retorna la instruccin select</returns>
        public DataSet ExecuteDataSet(string dataSetName, string sqlStatement) {
            DataSet ds = new DataSet(dataSetName);
            using (SqlCommand cmd = new SqlCommand(sqlStatement, this.conn)) {
                new SqlDataAdapter(cmd).Fill(ds);
            }
            return ds;
        }

        #region Metodos privados de soporte para la clase
        private object ExecuteScalar(string sqlStatement) {
            using (SqlCommand cmd = new SqlCommand(sqlStatement, conn)) {
                return cmd.ExecuteScalar();
            }
        }

        public string formatValue(DataRow dr, TableColumnInfo dc) {
            if ((dr[dc.Name] == null) || (dr[dc.Name] == DBNull.Value))
                return "NULL";

            if (!dc.IsSupportedType)
                return "NULL";

            if (dc.IsBit)
                return ((bool)dr[dc.Name] ? "1" : "0");

            if (dc.IsString)
                return string.Format(dc.ValueFormatString(), dr[dc.Name].ToString().Replace("'", "''"));

            if (dc.IsNumeric)
                return string.Format(dc.ValueFormatString(), dr[dc.Name].ToString());

            if (dc.IsDate) {
                DateTime time = (DateTime)dr[dc.Name];
                return string.Format(dc.ValueFormatString(),
                                     time.Day.ToString(),
                                     time.Month.ToString(),
                                     time.Year.ToString(),
                                     time.Hour.ToString(),
                                     time.Minute.ToString(),
                                     time.Second.ToString());
            }

            if (dc.IsXML)
                return string.Format(dc.ValueFormatString(), dr[dc.Name].ToString().Replace("'", "''"));

            return dr[dc.Name].ToString();
        }

        private string ColumnList(List<TableColumnInfo> columnas) {
            string s = "";

            foreach (TableColumnInfo info in columnas) {
                s = s + string.Format("[{0}], ", info.Name);
            }

            if (s != "") s = s.Substring(0, s.Length - 2);

            return s;
        }
        #endregion

        #region IDisposable Members
        public void Dispose() {
            if (conn.State == ConnectionState.Open)
                conn.Close();
            conn.Dispose();
        }
        #endregion

        public override string ToString() {
            if (this._connectionString.Trim().Length == 0) return "";

            SqlConnectionStringBuilder builder = new SqlConnectionStringBuilder(this._connectionString);
            return string.Format("{0}.{1}", builder.DataSource, builder.InitialCatalog);
        }

        public string DataSource {
            get {
                SqlConnectionStringBuilder builder = new SqlConnectionStringBuilder(this._connectionString);
                return builder.DataSource;
            }
        }

        public string InitialCatalog {
            get {
                SqlConnectionStringBuilder builder = new SqlConnectionStringBuilder(this._connectionString);
                return builder.InitialCatalog;
            }
        }

        public List<string> getTablesOrderedByDependency(string[] unorderTableNames) {
            // inicializa las colecciones
            this.uniqueTableList.Clear();
            this.orderTables.Clear();
            this.restrictTableList.Clear();

            // Inicializa la conexin al servidor sql server
            DependencyWalker walker = new DependencyWalker(server);

            List<Table> unorderTables = new List<Table>();
            foreach (string tbl in unorderTableNames) {
                this.restrictTableList.Add(tbl.Substring(tbl.IndexOf(".") + 1), "");
                unorderTables.Add(sqlDB.Tables[tbl.Substring(tbl.IndexOf(".") + 1)]);
            }

            // Obtiene el rbol de dependencias de las tabla especificadas en el arreglo
            DependencyTree tree = walker.DiscoverDependencies(unorderTables.ToArray(), true);
            DependencyTreeNode rootNode = tree.FirstChild;
            while (rootNode != null) {
                // Solo analiza dependencias de tablas
                if (rootNode.Urn.Type == "Table") {
                    // Llama al procedimiento que agrega los nodos hijos de manera recursiva
                    this.AddDependencyChildrenNode(rootNode);

                    // Crea un objeto tipo tabla para insertarlo an queue
                    Table table = (Table)server.GetSmoObject(rootNode.Urn);
                    if (!this.uniqueTableList.Contains(table.Name) && this.restrictTableList.Contains(table.Name)) {
                        this.uniqueTableList.Add(table.Name, "");
                        this.orderTables.Enqueue(table);
                    }
                }
                rootNode = rootNode.NextSibling;
            }

            // Del queue obtiene la lista ordenada de acuerdo a la dependencia
            List<string> orderTableNames = new List<string>();
            while (this.orderTables.Count > 0) {
                Table tbl = this.orderTables.Dequeue();
                orderTableNames.Add(string.Format("{0}.{1}", tbl.Schema, tbl.Name));
            }
            return orderTableNames;
        }

        private void AddDependencyChildrenNode(DependencyTreeNode node) {
            DependencyTreeNode child = node.FirstChild;

            while (child != null) {
                // Solo analiza dependencias de tablas
                if (child.Urn.Type == "Table") {
                    // Llama al procedimiento que agrega los nodos hijos de manera recursiva
                    this.AddDependencyChildrenNode(child);

                    // Crea un objeto tipo tabla para insertarlo an queue
                    Table table = (Table)server.GetSmoObject(child.Urn);
                    if (!this.uniqueTableList.Contains(table.Name) && this.restrictTableList.Contains(table.Name)) {
                        this.uniqueTableList.Add(table.Name, "");
                        this.orderTables.Enqueue(table);
                    }
                }
                child = child.NextSibling;
            }
        }
    }

    public class TableColumnInfo {
        public string Name = "";
        public string Type = "";
        public bool IsPrimaryKey;

        public TableColumnInfo() { }

        public TableColumnInfo(string name, string type, bool isPrimaryKey) {
            this.Name = name;
            this.Type = type;
            this.IsPrimaryKey = isPrimaryKey;
        }

        public string ValueFormatString() {
            if (!this.IsSupportedType)
                return "";

            if (this.IsBit || this.IsNumeric)
                return "{0}";

            if (this.IsDate) {
                if (this.Type.ToLower() == "datetime")
                    return "convert(datetime, '{2:0000}.{1:00}.{0:00} {3:00}:{4:00}:{5:00}', 120)";
                else
                    return "convert(smalldatetime, '{2:0000}.{1:00}.{0:00}', 102)";
            }

            if (this.IsXML)
                return "'{0}'";

            return "'{0}'";
        }

        public bool IsBit {
            get {
                return (this.Type.ToLower() == "bit");
            }
        }

        public bool IsDate {
            get {
                return this.Type.ToLower() == "datetime" || this.Type.ToLower() == "smalldatetime";
            }
        }

        public bool IsNumeric {
            get {
                switch (this.Type.ToLower()) {
                    case "bigint":
                    case "float":
                    case "int":
                    case "money":
                    case "smallmoney":
                    case "real":
                    case "smallint":
                    case "tinyint":
                    case "decimal":
                    case "numeric":
                        return true;
                }
                return false;
            }
        }

        public bool IsString {
            get {
                return this.IsSupportedType && !this.IsDate && !this.IsNumeric && !this.IsXML && !this.IsBit;
            }
        }

        public bool IsSupportedType {
            get {
                switch (this.Type.ToLower()) {
                    case "bigint":
                    case "bit":
                    case "float":
                    case "int":
                    case "money":
                    case "smallmoney":
                    case "real":
                    case "smallint":
                    case "tinyint":
                    case "decimal":
                    case "numeric":
                    case "varchar":
                    case "nvarchar":
                    case "text":
                    case "ntext":
                    case "char":
                    case "nchar":
                    case "datetime":
                    case "smalldatetime":
                    case "xml":
                        return true;
                }
                return false;
            }
        }

        public bool IsXML {
            get {
                return (this.Type.ToLower() == "xml");
            }
        }
    }
}
