using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;
using SyncDBMSSQL.Properties;

namespace SyncDBMSSQL {
    public partial class frmMain : Form {

        // Variables para controlar la sincronizacin de la base origen
        private string csSource = "";
        private bool _sourceNeedTest = true;
        private DataAccessLayer dbSource;

        // Variables para controlar la sincronizacin de la base destino
        private string csTarget = "";
        private bool _targetNeedTest = true;
        private DataAccessLayer dbTarget;

        // Crea variables para almacenar los scripts generados
        private StringBuilder insSQL = new StringBuilder();
        private StringBuilder delSQL = new StringBuilder();
        private StringBuilder updSQL = new StringBuilder();

        private bool CancelarComparacion = false;

        #region Propiedades
        private bool sourceNeedTest {
            get { return _sourceNeedTest; }
            set {
                _sourceNeedTest = value;
                EnableControls();
            }
        }

        private bool targetNeedTest {
            get { return _targetNeedTest; }
            set {
                _targetNeedTest = value;
                EnableControls();
            }
        }
        #endregion

        public frmMain() {
            InitializeComponent();

            frmAbout about = new frmAbout();
            this.Text = about.AssemblyTitle;
            lnkAbout.Text = about.AssemblyTitle + "\n" + about.AssemblyCompany + ". " + about.AssemblyCopyright;
            about = null;

            SqlConnectionStringBuilder csb;
            EnableControls();

            // Inicializa la conexin que pudiera estar almacenada en los settings del usuario
            btTestSource.Enabled = false;
            csSource = Settings.Default.SourceConnectionString;
            if (csSource.Trim().Length > 0) {
                csb = new SqlConnectionStringBuilder(this.csSource);
                txtHostSource.Text = csb.DataSource;
                txtDBSource.Text = csb.InitialCatalog;
                btTestSource.Enabled = true;
            }

            // Inicializa la conexin que pudiera estar almacenada en los settings del usuario
            btTestTarget.Enabled = false;
            csTarget = Settings.Default.TargetConnectionString;
            if (csTarget.Trim().Length > 0) {
                csb = new SqlConnectionStringBuilder(this.csTarget);
                txtHostTarget.Text = csb.DataSource;
                txtDBTarget.Text = csb.InitialCatalog;
                btTestTarget.Enabled = true;
            }

            // Inicializa la lista de patrones excluyentes de columnas almacenada en los settigns del usuario
            setExcludePatternList(lbExcludePatterns.Items);

            Tabs.SelectedIndexChanged += new EventHandler(Tabs_SelectedIndexChanged);

            btClearPattern.Enabled = lbExcludePatterns.Items.Count > 0;
            if (lbExcludePatterns.Items.Count > 0) {
                lbExcludePatterns.SelectedIndex = 0;
                lbExcludePatterns_SelectedIndexChanged(lbExcludePatterns, null);
            }
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e) {
            if (!btTestSource.Enabled) {
                Settings.Default.SourceConnectionString = csSource;
            }

            if (!btTestTarget.Enabled) {
                Settings.Default.TargetConnectionString = csTarget;
            }

            if (lbExcludePatterns.Items.Count > 0)
                Settings.Default.ExcludeColumnPattern = getExcludePatternList();
            else
                Settings.Default.ExcludeColumnPattern = "";

            Settings.Default.Save();

            if (dbSource != null)
                dbSource.Dispose();
            if (dbTarget != null)
                dbTarget.Dispose();
        }

        #region Sincronizacion de controles para definir la Source Database
        private void btConSource_Click(object sender, EventArgs e) {
            frmChangeConnection frm = new frmChangeConnection();
            frm.Text = "Configurar Conexin a Base de Datos Origen";
            frm.ConnectionString = this.csSource;

            if (frm.ShowDialog() == DialogResult.OK) {
                csSource = frm.ConnectionString;
                SqlConnectionStringBuilder csb = new SqlConnectionStringBuilder(csSource);
                txtHostSource.Text = csb.DataSource;
                txtDBSource.Text = csb.InitialCatalog;
                btTestSource_Click(btTestSource, null);
            }

            frm = null;
        }

        private void btTestSource_Click(object sender, EventArgs e) {
            try {
                this.Cursor = Cursors.WaitCursor;

                dbSource = new DataAccessLayer(csSource);
                sbLblSource.Text = "Origen: " + dbSource.ToString();
                sourceNeedTest = false;
                fillTableList();

            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message, "Error de Conexin a la base de datos");
                sourceNeedTest = true;
                sbLblSource.Text = "";
            }

            this.Cursor = Cursors.Default;
        }

        private void fillTableList() {
            if (!sourceNeedTest && !targetNeedTest) {
                lvTables.Items.Clear();
                List<string> tablas = dbSource.getTables();
                foreach (string tbl in tablas)
                    lvTables.Items.Add(tbl);
            }
        }
        #endregion

        #region Sincronizacion de controles para definir la Target Database
        private void btConTarget_Click(object sender, EventArgs e) {
            frmChangeConnection frm = new frmChangeConnection();
            frm.Text = "Configurar Conexin a Base de Datos Destino";
            frm.ConnectionString = this.csTarget;

            if (frm.ShowDialog() == DialogResult.OK) {
                csTarget = frm.ConnectionString;
                SqlConnectionStringBuilder csb = new SqlConnectionStringBuilder(csTarget);
                txtHostTarget.Text = csb.DataSource;
                txtDBTarget.Text = csb.InitialCatalog;
                btTestTarget_Click(btTestTarget, null);
            }
            frm = null;
        }

        private void btTestTarget_Click(object sender, EventArgs e) {
            try {
                this.Cursor = Cursors.WaitCursor;

                dbTarget = new DataAccessLayer(csTarget);
                sbLblTarget.Text = "Destino: " + dbTarget.ToString();
                targetNeedTest = false;
                fillTableList();

            }
            catch (Exception ex) {
                MessageBox.Show(ex.Message, "Error de Conexin a la base de datos");
                targetNeedTest = true;
                sbLblTarget.Text = "";
            }

            this.Cursor = Cursors.Default;
        }
        #endregion

        #region Metodos Privados
        private bool Compare(object sourceValue, object targetValue) {
            if (isDBNull(sourceValue) && isDBNull(targetValue))
                return true;
            else if (isDBNull(sourceValue) || isDBNull(targetValue))
                return false;
            else
                return (sourceValue.ToString() == targetValue.ToString());
        }

        private bool isDBNull(object value) {
            return (value == null || value == DBNull.Value);
        }

        private void EnableControls() {
            tsbComparar.Enabled = !(sourceNeedTest || targetNeedTest);
            btTestSource.Enabled = sourceNeedTest;
            btTestTarget.Enabled = targetNeedTest;
        }
        #endregion

        #region Maneja los chequeos de tablas seleccionadas
        private void seleccionarTodosToolStripMenuItem_Click(object sender, EventArgs e) {
            if (lvTables.Items.Count > 0)
                for (int i = 0; i <= lvTables.Items.Count - 1; i++)
                    lvTables.SetItemChecked(i, true);
        }

        private void ningunoToolStripMenuItem_Click(object sender, EventArgs e) {
            if (lvTables.Items.Count > 0)
                for (int i = 0; i <= lvTables.Items.Count - 1; i++)
                    lvTables.SetItemChecked(i, false);
        }

        private void lvTables_ItemCheck(object sender, ItemCheckEventArgs e) {
            EnableControls();
        }
        #endregion

        #region Eventos de servicio para los botones del toolbar
        private void tbsSave_Click(object sender, EventArgs e) {
            if (Tabs.SelectedIndex >= 1 && Tabs.SelectedIndex <= 3) {
                saveDLG.DefaultExt = "sql";
                saveDLG.Filter = "Archivos SQL|*.sql|Archivos Texto|*.txt|Todos los Archivos|*.*";
                saveDLG.Title = "Salvar el script generado";
            }
            else {
                saveDLG.DefaultExt = "txt";
                saveDLG.Filter = "Archivos Texto|*.txt|Todos los Archivos|*.*";
                saveDLG.Title = "Salvar el archivo de errores generado";
            }

            if (saveDLG.ShowDialog() == DialogResult.OK) {
                StreamWriter sw = new StreamWriter(saveDLG.FileName);
                switch (Tabs.SelectedIndex) {
                    case 1:
                        sw.Write(insSQL.ToString());
                        break;

                    case 2:
                        sw.Write(updSQL.ToString());
                        break;

                    case 3:
                        sw.Write(delSQL.ToString());
                        break;

                    case 4:
                        sw.Write(txtErrores.Text);
                        break;

                    default:
                        break;
                }
                sw.Close();
            }
        }

        private void tsbCancelar_Click(object sender, EventArgs e) {
            CancelarComparacion = true;
        }

        private void tsbComparar_Click(object sender, EventArgs e) {
            if (lvTables.CheckedItems.Count == 0) {
                MessageBox.Show("Debe seleccionar al menos una tabla para realizar la comparacin.", "Comparacin");
                if (Tabs.SelectedIndex != 0) {
                    Tabs.SelectedIndex = 0;
                    Tabs_SelectedIndexChanged(Tabs, System.EventArgs.Empty);
                }
                lvTables.Focus();
                return;
            }

            if (!(chkGenInserts.Checked || chkGenUpdates.Checked || chkGenDelete.Checked)) {
                MessageBox.Show("Debe seleccionar al menos una operacin de direccin de comparacin que realizar.", "Comparacin");
                if (Tabs.SelectedIndex != 0) {
                    Tabs.SelectedIndex = 0;
                    Tabs_SelectedIndexChanged(Tabs, System.EventArgs.Empty);
                }
                chkGenInserts.Focus();
                return;
            }

            dbSource.CloseConnection();
            dbSource.OpenConnection();

            dbTarget.CloseConnection();
            dbTarget.OpenConnection();

            CancelarComparacion = false;

            StringBuilder errors = new StringBuilder();

            insSQL = new StringBuilder(string.Format("-- Registros que existen en {0} y no existen en {1}", dbSource.ToString(), dbTarget.ToString()));
            insSQL.AppendLine();

            insSQL.AppendLine();
            insSQL.AppendFormat("USE [{0}]", dbTarget.InitialCatalog);
            insSQL.AppendLine();
            insSQL.AppendLine("GO");
            insSQL.AppendLine();
            insSQL.AppendLine("BEGIN TRANSACTION");

            delSQL = new StringBuilder(string.Format("-- Registros que existen en {1} y no existen en {0}", dbSource.ToString(), dbTarget.ToString()));
            delSQL.AppendLine();

            delSQL.AppendLine();
            delSQL.AppendFormat("USE [{0}]", dbTarget.InitialCatalog);
            delSQL.AppendLine();
            delSQL.AppendLine("GO");
            delSQL.AppendLine();
            delSQL.AppendLine("BEGIN TRANSACTION");

            updSQL = new StringBuilder("-- Registros diferentes entre las bases de datos --");
            updSQL.AppendLine();

            updSQL.AppendLine();
            updSQL.AppendFormat("USE [{0}]", dbTarget.InitialCatalog);
            updSQL.AppendLine();
            updSQL.AppendLine("GO");
            updSQL.AppendLine();
            updSQL.AppendLine("BEGIN TRANSACTION");

            // Elimina informacin de los controles visuales
            txtInserts.Text = "";
            txtUpdates.Text = "";
            txtDeletes.Text = "";
            txtErrores.Text = "";

            // Loop por las tablas a comparar
            sbLblProg.Visible = true;
            tsbCancelar.Visible = true;
            tsbComparar.Enabled = false;
            grpSourceDB.Enabled = false;
            grpTargetDB.Enabled = false;
            grpOpciones.Enabled = false;

            sbProg.Value = 0;
            sbProg.Minimum = 0;
            if (chkGenDelete.Checked && (chkGenInserts.Checked || chkGenUpdates.Checked)) {
                sbProg.Maximum = lvTables.CheckedItems.Count * 2;
            }
            else {
                sbProg.Maximum = lvTables.CheckedItems.Count;
            }
            sbProg.Visible = true;

            string[] unorderTableNames = new string[this.lvTables.CheckedItems.Count];
            lvTables.CheckedItems.CopyTo(unorderTableNames, 0);
            List<string> orderTableNames = this.dbSource.getTablesOrderedByDependency(unorderTableNames);

            if (chkGenDelete.Checked && !CancelarComparacion) {
                // Crea una lista con los nombres de tablas ordenados descendentemente por dependencia
                // para crear el script de los deletes sobre ella
                List<string> reverseOrderTableNames = new List<string>();
                foreach (string tbl in orderTableNames) {
                    reverseOrderTableNames.Insert(0, tbl);
                }

                foreach (string tbl in reverseOrderTableNames) {
                    if (this.CancelarComparacion) break;

                    string tblName = tbl.Substring(tbl.IndexOf(".") + 1);
                    string tblSchema = tbl.Substring(0, tbl.IndexOf("."));

                    delSQL.AppendLine();
                    delSQL.AppendFormat("-- {0}.{1}", dbSource.ToString(), tbl);
                    delSQL.AppendLine();

                    this.sbProg.Value++;
                    Application.DoEvents();

                    string sql = "";

                    // (1) verifica que exista la tabla en la base destino
                    if (dbTarget.ExisteTabla(tbl)) {
                        // (2) verifica que existan registros que deben insertarse en el destino

                        // Obtiene una coleccion de columnas de la tabla origen
                        List<TableColumnInfo> columnas = this.dbSource.getTableColumns(tbl);
                        if (columnas.Count > 0) {
                            // lista de campos con las llaves primarias
                            sql = "";
                            foreach (TableColumnInfo col in columnas)
                                if (col.IsPrimaryKey)
                                    sql = sql + string.Format("isnull(cast([{0}] as varchar(max)), 'null')+'|'+", col.Name);

                            if (sql != "")
                                sql = sql.Substring(0, sql.Length - 1);

                            if (sql != "") {
                                // Obtiene los datos de las dos tablas en origen y destino
                                string sqlSel = string.Format("select {2} from [{0}].[{1}] order by {2}", tblSchema, tblName, sql);
                                DataSet dSource = dbSource.ExecuteDataSet(sqlSel);
                                DataSet dTarget = dbTarget.ExecuteDataSet(sqlSel);

                                // Hace un loop por los registros de origen para buscarlos en destino
                                if (dSource != null && dSource.Tables.Count > 0 &&
                                    dTarget != null && dTarget.Tables.Count > 0) {

                                    foreach (DataRow drt in dTarget.Tables[0].Rows) {
                                        if (this.CancelarComparacion) break;
                                        Application.DoEvents();

                                        bool found = false;
                                        foreach (DataRow drs in dSource.Tables[0].Rows) {
                                            if (this.CancelarComparacion) break;
                                            Application.DoEvents();

                                            found = (drs[0].ToString() == drt[0].ToString());
                                            if (found) break;
                                        }
                                        if (!found) {
                                            this.delSQL.AppendFormat("delete from [{0}].[{1}] where {2} = '{3}'{4}", tblSchema, tblName, sql, drt[0].ToString(), Settings.Default.EndOfLineCharacter);
                                            this.delSQL.AppendLine();
                                        }
                                    }
                                }
                            }
                            else {
                                errors.AppendLine(string.Format("No es posible comparar tablas sin llave primaria ({1}.{0})", tbl, dbSource.ToString()));
                            }
                        }
                        else {
                            errors.AppendLine(string.Format("No fue posible obtener la descripcin de columnas de la tabla {1}.{0} en la base origen", tbl, dbSource.ToString()));
                        }
                    }
                    else {
                        errors.AppendLine(string.Format("La tabla {1}.{0} no fue hallada en la base de datos destino", tbl, dbTarget.ToString()));
                    }
                }
            }

            if ((chkGenInserts.Checked || chkGenUpdates.Checked) && !CancelarComparacion) {
                foreach (string tbl in orderTableNames) {
                    if (CancelarComparacion) break;

                    string tblName = tbl.Substring(tbl.IndexOf(".") + 1);
                    string tblSchema = tbl.Substring(0, tbl.IndexOf("."));

                    insSQL.AppendLine();
                    insSQL.AppendFormat("-- {0}.{1}", dbSource.ToString(), tbl);
                    insSQL.AppendLine();
                    if (dbTarget.TableHasIdentityColumn(tbl)) {
                        insSQL.AppendFormat("SET IDENTITY_INSERT {0} ON{1}", tbl, Settings.Default.EndOfLineCharacter);
                        insSQL.AppendLine();
                    }

                    updSQL.AppendLine();
                    updSQL.AppendFormat("-- {0}.{1}", dbSource.ToString(), tbl);
                    updSQL.AppendLine();

                    sbProg.Value++;
                    Application.DoEvents();

                    string sql = "";

                    // (1) verifica que exista la tabla en la base destino
                    if (dbTarget.ExisteTabla(tbl)) {
                        // (2) verifica que existan registros que deben insertarse en el destino

                        // Obtiene una coleccion de columnas de la tabla origen
                        List<TableColumnInfo> columnas = dbSource.getTableColumns(tbl);
                        if (columnas.Count > 0) {
                            // lista de campos con las llaves primarias
                            sql = "";
                            foreach (TableColumnInfo col in columnas)
                                if (col.IsPrimaryKey)
                                    sql = sql + string.Format("isnull(cast([{0}] as varchar(max)), 'null')+'|'+", col.Name);

                            if (sql != "")
                                sql = sql.Substring(0, sql.Length - 1);

                            if (sql != "") {
                                // Obtiene los datos de las dos tablas en origen y destino
                                string sqlSel = string.Format("select {2} from [{0}].[{1}] order by {2}", tblSchema, tblName, sql);
                                DataSet dSource = dbSource.ExecuteDataSet(sqlSel);
                                DataSet dTarget = dbTarget.ExecuteDataSet(sqlSel);

                                // Hace un loop por los registros de origen para buscarlos en destino
                                if (dSource != null && dSource.Tables.Count > 0 &&
                                    dTarget != null && dTarget.Tables.Count > 0) {

                                    if (chkGenInserts.Checked) {
                                        // (1) loop por los registros para generar instrucciones Insert
                                        foreach (DataRow drs in dSource.Tables[0].Rows) {
                                            if (CancelarComparacion) break;
                                            Application.DoEvents();

                                            bool found = false;

                                            foreach (DataRow drt in dTarget.Tables[0].Rows) {
                                                if (CancelarComparacion) break;
                                                Application.DoEvents();

                                                found = (drs[0].ToString() == drt[0].ToString());
                                                if (found) break;
                                            }

                                            if (!found) {
                                                insSQL.AppendLine(this.dbSource.generateInsert(tbl, columnas, sql, drs[0].ToString()));
                                            }
                                        }
                                    }

                                    if (chkGenUpdates.Checked) {
                                        // (4) genera instrucciones update para los registros diferentes
                                        //loop por los registros
                                        foreach (DataRow drs in dSource.Tables[0].Rows) {
                                            if (CancelarComparacion) break;
                                            Application.DoEvents();

                                            foreach (DataRow drt in dTarget.Tables[0].Rows)
                                                if (drs[0].ToString() == drt[0].ToString()) {
                                                    DataRow sourceRow = dbSource.getDataRow(tbl, columnas, sql, drs[0].ToString());
                                                    DataRow targetRow = dbTarget.getDataRow(tbl, columnas, sql, drs[0].ToString());

                                                    string u = "";
                                                    string w = "";
                                                    if (sourceRow != null && targetRow != null)
                                                        foreach (TableColumnInfo dc in columnas)
                                                            if (dc.IsPrimaryKey)
                                                                w += string.Format("[{0}] = {1} and ", dc.Name, this.dbSource.formatValue(sourceRow, dc));
                                                            else
                                                                if (ComparaColumna(dc.Name) && !Compare(sourceRow[dc.Name], targetRow[dc.Name]))
                                                                    u += string.Format("[{0}] = {1}, ", dc.Name, this.dbSource.formatValue(sourceRow, dc));
                                                    
                                                    if (u != "") {
                                                        u = u.Substring(0, u.Length - 2);
                                                        if (w != "")
                                                            w = w.Substring(0, w.Length - 5);

                                                        updSQL.AppendFormat(string.Format("update [{0}].[{1}] set {2} where {3}{4}", tblSchema, tblName, u, w, Settings.Default.EndOfLineCharacter));
                                                        updSQL.AppendLine();
                                                    }
                                                }

                                        }
                                    }
                                }
                            }
                            else {
                                errors.AppendLine(string.Format("No es posible comparar tablas sin llave primaria ({1}.{0})", tbl, dbSource.ToString()));
                            }
                        }
                        else {
                            errors.AppendLine(string.Format("No fue posible obtener la descripcin de columnas de la tabla {1}.{0} en la base origen", tbl, dbSource.ToString()));
                        }
                    }
                    else {
                        errors.AppendLine(string.Format("La tabla {1}.{0} no fue hallada en la base de datos destino)", tbl, dbTarget.ToString()));
                    }

                    if (dbTarget.TableHasIdentityColumn(tbl)) {
                        insSQL.AppendFormat("SET IDENTITY_INSERT {0} OFF{1}", tbl, Settings.Default.EndOfLineCharacter);
                        insSQL.AppendLine();
                    }
                }
            }

            insSQL.AppendLine();
            delSQL.AppendLine();
            updSQL.AppendLine();

            insSQL.AppendLine("COMMIT TRANSACTION");
            updSQL.AppendLine("COMMIT TRANSACTION");
            delSQL.AppendLine("COMMIT TRANSACTION");
  
            txtInserts.Text = (chkGenInserts.Checked ? insSQL.ToString() : "");
            txtUpdates.Text = (chkGenUpdates.Checked ? updSQL.ToString() : "");
            txtDeletes.Text = (chkGenDelete.Checked ? delSQL.ToString() : "");
            txtErrores.Text = errors.ToString();

            sbLblProg.Visible = false;
            sbProg.Visible = false;

            tsbCancelar.Visible = false;
            tsbComparar.Enabled = true;
            grpSourceDB.Enabled = true;
            grpTargetDB.Enabled = true;
            grpOpciones.Enabled = true;

            Tabs.SelectedTab = tabInserts;
            Tabs_SelectedIndexChanged(Tabs, System.EventArgs.Empty);
            txtInserts.SelectionLength = 0;
        }

        private bool ComparaColumna(string p) {
            foreach (string s in lbExcludePatterns.Items)
                if (p.ToUpper().Contains(s.ToUpper()))
                    return false;
            return true;
        }
        #endregion

        #region Mtodos para manejar la lista de patrones para excluir columnas en la comparacin
        private string getExcludePatternList() {
            string l = "";
            foreach (string s in lbExcludePatterns.Items)
                l += s + "|";
            if (l.EndsWith("|")) l = l.Substring(0, l.Length - 1);
            return l;
        }

        private void setExcludePatternList(ListBox.ObjectCollection objectCollection) {
            string[] l = Settings.Default.ExcludeColumnPattern.Split('|');
            foreach (string s in l)
                if (s.Trim().Length > 0) objectCollection.Add(s.Trim());
        }

        private void lbExcludePatterns_SelectedIndexChanged(object sender, EventArgs e) {
            btDelPattern.Enabled = lbExcludePatterns.SelectedIndex >= 0;
            btClearPattern.Enabled = lbExcludePatterns.Items.Count > 0;
        }

        private void btAddPattern_Click(object sender, EventArgs e) {
            if (txtNewPattern.Text.Trim().Length > 0) {
                lbExcludePatterns.Items.Add(txtNewPattern.Text);
                lbExcludePatterns.SelectedIndex = lbExcludePatterns.Items.Count - 1;
                txtNewPattern.Text = "";
                txtNewPattern.Focus();
            }
            else {
                MessageBox.Show("Debe incluir un texto para que sirve de patron de filtro de columnas", "Excluir Patrn", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                txtNewPattern.Focus();
            }
        }

        private void btDelPattern_Click(object sender, EventArgs e) {
            if (lbExcludePatterns.SelectedIndex >= 0) {
                int pi = lbExcludePatterns.SelectedIndex;
                lbExcludePatterns.Items.RemoveAt(lbExcludePatterns.SelectedIndex);
                if (lbExcludePatterns.Items.Count > 0) {
                    if (pi >= 0 && pi <= lbExcludePatterns.Items.Count - 1)
                        lbExcludePatterns.SelectedIndex = pi;
                    else if (pi > lbExcludePatterns.Items.Count - 1)
                        lbExcludePatterns.SelectedIndex = lbExcludePatterns.Items.Count - 1;
                }
            }
            lbExcludePatterns_SelectedIndexChanged(lbExcludePatterns, null);
        }

        private void btClearPattern_Click(object sender, EventArgs e) {
            lbExcludePatterns.Items.Clear();
            lbExcludePatterns_SelectedIndexChanged(lbExcludePatterns, null);
        }

        private void txtNewPattern_KeyUp(object sender, KeyEventArgs e) {
            if (e.KeyCode == Keys.Enter)
                btAddPattern_Click(btAddPattern, null);
        }
        #endregion

        void Tabs_SelectedIndexChanged(object sender, EventArgs e) {
            tbsSave.Enabled = bool.Parse(Tabs.SelectedTab.Tag.ToString());
        }

        private void lnkSelectAll_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) {
            seleccionarTodosToolStripMenuItem_Click(seleccionarTodosToolStripMenuItem, null);
        }

        private void lnkSelectNone_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) {
            ningunoToolStripMenuItem_Click(ningunoToolStripMenuItem, null);
        }

        private void lnkAbout_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) {
            frmAbout fa = new frmAbout();
            fa.ShowDialog();
        }

    }
}
